/*
 * main.c
 * Kick off program.
 *
 * Copyright (c) 1996 Systems Architecture Research Centre,
 *		   City University, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@sarc.city.ac.uk>, February 1996.
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "gtypes.h"
#include "access.h"
#include "object.h"
#include "constants.h"
#include "classMethod.h"
#include "baseClasses.h"
#include "thread.h"
#include "slib.h"

void initialise(void);
object* alloc_objectarray(int, char*);
stringClass* makeJavaString(char*, int);
long do_execute_java_class_method(char*, char*, char*, ...);
void killThread(thread*);

static int options(char**);

#define	MAIN		"main"
#define	MAINSIG		"([Ljava/lang/String;)V"
#define	STRINGSIG	"Ljava/lang/String;"

/*
 * MAIN
 */
int
main(int argc, char* argv[])
{
	classes* class;
	void* func;
	object* args;
	stringClass** str;
	int i;
	int farg;

	/* Process program options */
	farg = options(argv);

	/* Get the class name to start with */
	if (argv[farg] == 0) {
		fprintf(stderr, "No class specified.\n");
		exit(1);
	}

	/* Initialise */
	initialise();

	/* Build an array of strings as the arguments */
	args = alloc_objectarray(argc-(farg+1), STRINGSIG);

	/* Build each string and put into the array */
	str = (stringClass**)(args+1);
	for (i = farg+1; i < argc; i++) {
		str[i-(farg+1)] = makeJavaString(argv[i], strlen(argv[i]));
	}

	/* Kick it */
	do_execute_java_class_method(argv[farg], MAIN, MAINSIG, args);
	killThread(0);
	/* This should never return */
	exit(1);
	return(1);
}

/*
 * Process program's flags.
 */
static
int
options(char** argv)
{
	int i;

	for (i = 1; argv[i] != 0; i++) {
		if (argv[i][0] != '-') {
			break;
		}

		if (strcmp(argv[i], "-version") == 0) {
			fprintf(stderr, "Kaffe version %s ", KVER);
			fprintf(stderr, "(c) Copyright Tim Wilkinson\n");
			fprintf(stderr, "Systems Architecture Research Centre, City University, UK.\n");
		}
		else {
			fprintf(stderr, "Unknown flag: %s\n", argv[i]);
		}
	}

	/* Return first no-flag argument */
	return (i);
}
