/*
 * readClassConfig.h
 * Various bits of information in a Java class file.
 *
 * Copyright (c) 1996 Systems Architecture Research Centre,
 *		   City University, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@sarc.city.ac.uk>, February 1996.
 */

#ifndef __readclassconfig_h
#define __readclassconfig_h

#include <stdio.h>

typedef FILE classFile;
typedef struct _classes classes;
typedef struct _methods methods;

extern FILE* include;
extern FILE* stub;
extern char className[];
extern char superName[];

extern char* translateSig(char*, char**, int*);
extern char* translateSigType(char*, char*);

/*
 * Add a class.
 */
#define	ADDCLASS(this, super, access, constants)			\
	if (super != 0) {						\
		int s;							\
		char* snm;						\
		snm = (char*)constants->data[constants->data[super]];	\
		for (s = 0; snm[s] != 0; s++) {				\
			if (snm[s] == '/') {				\
				superName[s] = '_';			\
			}						\
			else {						\
				superName[s] = snm[s];			\
			}						\
		}							\
		if (strcmp(superName, "java_lang_Object") == 0) {	\
			superName[0] = 0;				\
		}							\
	}

/*
 * Read and process a field.
 */
#define	READFIELD_START()						\
	fprintf(include, "/* DO NOT EDIT THIS FILE - it is machine generated */\n");\
	fprintf(include, "#include <native.h>\n");			\
	fprintf(include, "/* Header for class %s */\n", className);	\
	fprintf(include, "\n");						\
	fprintf(include, "#ifndef _Included_%s\n", className);		\
	fprintf(include, "#define _Included_%s\n", className);		\
	fprintf(include, "\n");						\
	fprintf(include, "typedef struct Class%s {\n", className);	\
	if (superName[0] != 0) {					\
		fprintf(include, "  struct Class%s super;\n\n", superName);\
	}								\
	else {								\
		fprintf(include, "  object super;\n\n");		\
	}

#define	READFIELD_END()							\
	fprintf(include, "} Class%s;\n", className);			\
	fprintf(include, "HandleTo(%s);\n\n", className)

#define	READFIELD(fp, this)						\
	do {								\
		field_info f;						\
		int argsize = 0;					\
		char* arg;						\
		static int argpos = 0;					\
									\
		readu2(&f.access_flags, fp);				\
		readu2(&f.name_index, fp);				\
		readu2(&f.signature_index, fp);				\
									\
		/* Ignore statics */					\
		if (f.access_flags & ACC_STATIC) {			\
			continue;					\
		}							\
		arg = translateSig((char*)constant_pool->data[f.signature_index], 0, &argsize);\
		if (argpos % argsize == 1) {				\
			fprintf(include, "  int __dummy%d;\n", argpos);	\
			argpos++;					\
		}							\
		argpos += argsize;					\
		fprintf(include, "  %s", translateSig((char*)constant_pool->data[f.signature_index], 0, &argsize));\
		fprintf(include, " %s;\n", (char*)constant_pool->data[f.name_index]);\
	} while(0)

/*
 * Read and process a method.
 */
#define	READMETHOD(fp, this)						\
	do {								\
		method_info m;						\
		char* name;						\
		char* sig;						\
		char* str;						\
		char* ret;						\
		char* tsig;						\
		char type;						\
		int j;							\
		char rtype;						\
		int args;						\
		int isStatic;						\
									\
		readu2(&m.access_flags, fp);				\
		readu2(&m.name_index, fp);				\
		readu2(&m.signature_index, fp);				\
									\
		/* Only generate stubs for native methods */		\
		if (!(m.access_flags & ACC_NATIVE)) {			\
			continue;					\
		}							\
		args = 0;						\
		isStatic = 1;						\
		if (!(m.access_flags & ACC_STATIC)) {			\
			isStatic = 0;					\
		}							\
		/* Generate method prototype */				\
		name = (char*)constant_pool->data[m.name_index];	\
		sig = (char*)constant_pool->data[m.signature_index];	\
		ret = strchr(sig,')');					\
		ret++;							\
		fprintf(include, "extern %s", translateSig(ret, 0, 0));	\
		fprintf(include, " %s_%s(", className, name);		\
		str = sig + 1;						\
		if (!isStatic) {					\
			args++;						\
			fprintf(include, "struct H%s*", className);	\
			if (str[0] != ')') {				\
				fprintf(include, ", ");			\
			}						\
		}							\
		while (str[0] != ')') {					\
			fprintf(include, "%s", translateSig(str, &str, &args));\
			if (str[0] != ')') {				\
				fprintf(include, ", ");			\
			}						\
		}							\
		fprintf(include, ");\n");				\
		/* Generate method stub */				\
		fprintf(stub, "\n/* SYMBOL: %s %s%s */\n", className, name, sig);\
		translateSigType(ret, &rtype);				\
		fprintf(stub, "EXPORT(void)\n");				\
		fprintf(stub, "Kaffe_%s_%s_stub(stack_item* _P_)\n", className, name);\
		fprintf(stub, "{\n");					\
		fprintf(stub, "\textern %s", translateSig(ret, 0, 0));	\
		fprintf(stub, " %s_%s(", className, name);		\
		str = sig + 1;						\
		if (!isStatic) {					\
			fprintf(stub, "void*");				\
			if (str[0] != ')') {				\
				fprintf(stub, ", ");			\
			}						\
		}							\
		while (str[0] != ')') {					\
			tsig = translateSig(str, &str, 0);		\
			if (strncmp(tsig, "struct H", 8) == 0) {	\
				fprintf(stub, "void*");			\
			}						\
			else {						\
				fprintf(stub, "%s", tsig);		\
			}						\
			if (str[0] != ')') {				\
				fprintf(stub, ", ");			\
			}						\
		}							\
		fprintf(stub, ");\n");					\
		fprintf(stub, "\t");					\
		if (rtype != 'v') {					\
			fprintf(stub, "%s ret = ", translateSig(ret, 0, 0));\
		}							\
		fprintf(stub, "%s_%s(", className, name);		\
		str = sig + 1;						\
		j = args - 1;						\
		if (!isStatic) {					\
			fprintf(stub, "_P_[%d].p", j);			\
			if (str[0] != ')') {				\
				fprintf(stub, ", ");			\
			}						\
			j--;						\
		}							\
		for (; str[0] != ')'; j--) {				\
			str = translateSigType(str, &type);		\
			/* This is horrid - but basically just copy */	\
			/* the data one-for-one.		    */	\
			switch(type) {					\
			case 'f':					\
				fprintf(stub, "_P_[%d].f", j);		\
				break;					\
			case 'd':					\
				j--;					\
				fprintf(stub, "_P_[%d].d", j);		\
				break;					\
			case 'l':					\
				j--;					\
				fprintf(stub, "_P_[%d].l", j);		\
				break;					\
			case 'i':					\
				fprintf(stub, "_P_[%d].i", j);		\
				break;					\
			case 'p':					\
				fprintf(stub, "_P_[%d].p", j);		\
				break;					\
			case 'v':					\
				break;					\
			}						\
			if (str[0] != ')') {				\
				fprintf(stub, ", ");			\
			}						\
		}							\
		fprintf(stub, ");\n");					\
		switch (rtype) {					\
		case 'f':						\
			fprintf(stub, "\treturn_float(ret);\n");	\
			break;						\
		case 'd':						\
			fprintf(stub, "\treturn_double(ret);\n");	\
			break;						\
		case 'l':						\
			fprintf(stub, "\treturn_long(ret);\n");		\
			break;						\
		case 'i':						\
			fprintf(stub, "\treturn_int(ret);\n");		\
			break;						\
		case 'p':						\
			fprintf(stub, "\treturn_ref(ret);\n");		\
			break;						\
		case 'v':						\
			break;						\
		}							\
		fprintf(stub, "}\n");					\
	} while(0)

#endif
